@@ -10,11 +10,12 @@ if not os.path.isfile(configFile):
 from letmedoit import config
 
 from letmedoit.health_check import HealthCheck
-if not hasattr(config, "openaiApiKey") or not config.openaiApiKey:
+if not hasattr(config, "currentMessages"):
     HealthCheck.setBasicConfig()
-    HealthCheck.changeAPIkey()
-    HealthCheck.saveConfig()
-    print("Configurations updated!")
+    if not hasattr(config, "openaiApiKey") or not config.openaiApiKey:
+        HealthCheck.changeAPIkey()
+    config.saveConfig()
+    #print("Configurations updated!")
 HealthCheck.checkCompletion()
 
 import autogen, os, json, traceback
@@ -35,9 +36,19 @@ class AutoGenMath:
         #    api_version=None,
         #)
         oai_config_list = []
-        for model in ("gpt-4-turbo-preview", "gpt-4-0125-preview", "gpt-4-1106-preview", "gpt-3.5-turbo", "gpt-3.5-turbo-16k", "gpt-4", "gpt-4-32k"):
+        for model in HealthCheck.tokenLimits.keys():
             oai_config_list.append({"model": model, "api_key": config.openaiApiKey})
+        if not config.chatGPTApiModel in HealthCheck.tokenLimits:
+            oai_config_list.append({"model": config.chatGPTApiModel, "api_key": config.openaiApiKey})
         os.environ["OAI_CONFIG_LIST"] = json.dumps(oai_config_list)
+        """
+        Code execution is set to be run in docker (default behaviour) but docker is not running.
+        The options available are:
+        - Make sure docker is running (advised approach for code execution)
+        - Set "use_docker": False in code_execution_config
+        - Set AUTOGEN_USE_DOCKER to "0/False/no" in your environment variables
+        """
+        os.environ["AUTOGEN_USE_DOCKER"] = "False"
 
     def getResponse(self, math_problem):
         config_list = autogen.config_list_from_json(
@@ -97,11 +108,11 @@ class AutoGenMath:
         })
         prompts = Prompts()
         self.print(f"<{config.terminalCommandEntryColor1}>AutoGen Math Solver launched!</{config.terminalCommandEntryColor1}>")
-        self.print("[press 'ctrl+q' to exit]")
+        self.print(f"""[press '{str(config.hotkey_exit).replace("'", "")[1:-1]}' to exit]""")
         while True:
             self.print(f"<{config.terminalCommandEntryColor1}>New session started!</{config.terminalCommandEntryColor1}>")
             self.print(f"<{config.terminalCommandEntryColor1}>Enter a math problem below:</{config.terminalCommandEntryColor1}>")
-            self.print("[press 'ctrl+q' to exit]")
+            self.print(f"""[press '{str(config.hotkey_exit).replace("'", "")[1:-1]}' to exit]""")
             math_problem = prompts.simplePrompt(style=promptStyle)
             if math_problem == config.exit_entry:
                 break
