@@ -12,13 +12,13 @@ if not hasattr(config, "max_consecutive_auto_reply"):
     config.max_consecutive_auto_reply = 10
 
 from letmedoit.health_check import HealthCheck
-if not hasattr(config, "openaiApiKey") or not config.openaiApiKey:
+if not hasattr(config, "currentMessages"):
     HealthCheck.setBasicConfig()
-    HealthCheck.changeAPIkey()
-    HealthCheck.saveConfig()
-    print("Configurations updated!")
+    if not hasattr(config, "openaiApiKey") or not config.openaiApiKey:
+        HealthCheck.changeAPIkey()
+    config.saveConfig()
+    #print("Configurations updated!")
 HealthCheck.checkCompletion()
-HealthCheck.setPrint()
 
 import autogen, os, json, traceback, chromadb, re, zipfile, datetime, traceback
 from chromadb.config import Settings
@@ -40,9 +40,19 @@ class AutoGenRetriever:
         #    api_version=None,
         #)
         oai_config_list = []
-        for model in ("gpt-4-turbo-preview", "gpt-4-0125-preview", "gpt-4-1106-preview", "gpt-3.5-turbo", "gpt-3.5-turbo-16k", "gpt-4", "gpt-4-32k"):
+        for model in HealthCheck.tokenLimits.keys():
             oai_config_list.append({"model": model, "api_key": config.openaiApiKey})
+        if not config.chatGPTApiModel in HealthCheck.tokenLimits:
+            oai_config_list.append({"model": config.chatGPTApiModel, "api_key": config.openaiApiKey})
         os.environ["OAI_CONFIG_LIST"] = json.dumps(oai_config_list)
+        """
+        Code execution is set to be run in docker (default behaviour) but docker is not running.
+        The options available are:
+        - Make sure docker is running (advised approach for code execution)
+        - Set "use_docker": False in code_execution_config
+        - Set AUTOGEN_USE_DOCKER to "0/False/no" in your environment variables
+        """
+        os.environ["AUTOGEN_USE_DOCKER"] = "False"
 
     def getResponse(self, docs_path, message, auto=False):
         if not os.path.exists(docs_path):
@@ -169,7 +179,7 @@ class AutoGenRetriever:
                 config.max_consecutive_auto_reply = int(max_consecutive_auto_reply)
 
         self.print(f"<{config.terminalCommandEntryColor1}>AutoGen Retriever launched!</{config.terminalCommandEntryColor1}>")
-        self.print("[press 'ctrl+q' to exit]")
+        self.print(f"""[press '{str(config.hotkey_exit).replace("'", "")[1:-1]}' to exit]""")
         
         self.print(f"<{config.terminalCommandEntryColor1}>Enter document path below (file / folder):</{config.terminalCommandEntryColor1}>")
         self.print(f"""Supported formats: *.{", *.".join(TEXT_FORMATS)}""" + ", *.zip")
