@@ -1,10 +1,9 @@
 from letmedoit import config
 from letmedoit.health_check import HealthCheck
-if not hasattr(config, "exit_entry"):
+if not hasattr(config, "currentMessages"):
     HealthCheck.setBasicConfig()
-    HealthCheck.saveConfig()
-    print("Configurations updated!")
-HealthCheck.setPrint()
+    config.saveConfig()
+    #print("Configurations updated!")
 from letmedoit.utils.prompts import Prompts
 from letmedoit.utils.terminal_system_command_prompt import SystemCommandPrompt
 import platform
