@@ -19,10 +19,17 @@ from prompt_toolkit import print_formatted_text, HTML
 from prompt_toolkit import prompt
 from prompt_toolkit.filters import Condition
 from prompt_toolkit.key_binding import KeyBindings, merge_key_bindings, ConditionalKeyBindings
-from letmedoit.utils.prompt_shared_key_bindings import prompt_shared_key_bindings
-from letmedoit.utils.prompt_multiline_shared_key_bindings import prompt_multiline_shared_key_bindings
 from prompt_toolkit.clipboard.pyperclip import PyperclipClipboard
+from prompt_toolkit.application import run_in_terminal
+from letmedoit.utils.vlc_utils import VlcUtil
+from letmedoit.utils.tts_utils import TTSUtil
+from letmedoit.utils.config_essential import defaultSettings
 from pathlib import Path
+from PIL import Image
+import speech_recognition as sr
+# a dummy import line to resolve ALSA error display
+import sounddevice
+
 
 def check_openai_errors(func):
     def wrapper(*args, **kwargs):
@@ -71,13 +78,11 @@ class HealthCheck:
     # token limit
     # reference: https://platform.openai.com/docs/models/gpt-4
     tokenLimits = {
-        #"gpt-3.5-turbo-instruct": 4097,
-        "gpt-3.5-turbo": 4097,
+        "gpt-4-turbo-preview": 128000, # Returns a maximum of 4,096 output tokens.
+        "gpt-4-0125-preview": 128000, # Returns a maximum of 4,096 output tokens.
+        "gpt-4-1106-preview": 128000, # Returns a maximum of 4,096 output tokens.
+        "gpt-3.5-turbo": 16385, # Returns a maximum of 4,096 output tokens.
         "gpt-3.5-turbo-16k": 16385,
-        "gpt-4-turbo-preview": 128000,
-        "gpt-4-0125-preview": 128000,
-        "gpt-4-1106-preview": 128000, # official 128,000; but "This model supports at most 4096 completion tokens"; set 8192 here to work with LetMeDoIt AI dynamic token feature
-        #"gpt-4-vision-preview": 128,000, # used in plugin "analyze images"
         "gpt-4": 8192,
         "gpt-4-32k": 32768,
     }
@@ -85,35 +90,50 @@ class HealthCheck:
     models = tuple(tokenLimits.keys())
 
     @staticmethod
-    def setBasicConfig(): # minimum config to work with standalone scripts built with AutoGen
-        config.multilineInput = False
-        config.openaiApiKey = ''
-        config.chatGPTApiModel = 'gpt-3.5-turbo-16k'
-        config.chatGPTApiMaxTokens = 4000
-        config.chatGPTApiMinTokens = 256
-        config.llmTemperature = 0.8
-        config.max_consecutive_auto_reply = 10
-        config.exit_entry = '.exit'
-        config.cancel_entry = '.cancel'
-        config.terminalPromptIndicatorColor1 = "ansimagenta"
-        config.terminalPromptIndicatorColor2 = "ansicyan"
-        config.terminalCommandEntryColor1 = "ansiyellow"
-        config.terminalCommandEntryColor2 = "ansigreen"
-        config.terminalResourceLinkColor = "ansiyellow"
-        config.terminalHeadingTextColor = "ansigreen"
-        config.mouseSupport = False
-        config.embeddingModel = "paraphrase-multilingual-mpnet-base-v2"
-        config.max_agents = 5
-        config.max_group_chat_round = 12
-        config.max_consecutive_auto_reply = 10
-        config.includeIpInSystemMessage = False
-        config.wrapWords = True
-        config.pygments_style = ""
-        config.systemMessage_chatgpt = 'You are a helpful assistant.'
-        config.systemMessage_geminipro = 'You are a helpful assistant.'
-        config.systemMessage_palm2 = 'You are a helpful assistant.'
-        config.systemMessage_codey = 'You are an expert on coding.'
-        HealthCheck.setPrint()
+    def setBasicConfig():
+        if not hasattr(config, "setBasicConfigDone") or not config.setBasicConfigDone:
+            # package folder
+            config.letMeDoItAIFolder = packageFolder
+            config.excludeConfigList = []
+            # Default Settings
+            for key, value in defaultSettings:
+                if not hasattr(config, key):
+                    value = pprint.pformat(value)
+                    exec(f"""config.{key} = {value} """)
+            # tts
+            config.isVlcPlayerInstalled = VlcUtil.isVlcPlayerInstalled()
+            config.tts = False if not config.isVlcPlayerInstalled and not config.isPygameInstalled and not config.ttsCommand and not config.elevenlabsApi else True
+            # Google Credentials
+            if config.google_cloud_credentials and os.path.isfile(config.google_cloud_credentials):
+                os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = config.google_cloud_credentials
+            else:
+                files = HealthCheck.getLocalStorage()
+                gccfile1 = os.path.join(files, "credentials_google_cloud.json")
+                gccfile2 = os.path.join(files, "credentials_googleaistudio.json")
+                gccfile3 = os.path.join(files, "credentials_googletts.json")
+                # set required file
+                config.google_cloud_credentials_file = gccfile1
+
+                if os.path.isfile(gccfile1):
+                    config.google_cloud_credentials = gccfile1
+                elif os.path.isfile(gccfile2):
+                    config.google_cloud_credentials = gccfile2
+                elif os.path.isfile(gccfile3):
+                    config.google_cloud_credentials = gccfile3
+                else:
+                    config.google_cloud_credentials = ""
+                os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = config.google_cloud_credentials if config.google_cloud_credentials else ""
+                
+            # print functions
+            HealthCheck.setPrint()
+
+            config.setBasicConfigDone = True
+
+    @staticmethod
+    def showErrors():
+        trace = traceback.format_exc()
+        print(trace if config.developer else "Error encountered!")
+        return trace
 
     @staticmethod
     def spinning_animation(stop_event):
@@ -138,16 +158,108 @@ class HealthCheck:
         except:
             pass
 
+    @staticmethod
+    def is_valid_image_file(file_path):
+        try:
+            # Open the image file
+            with Image.open(file_path) as img:
+                # Check if the file format is supported by PIL
+                img.verify()
+                return True
+        except (IOError, SyntaxError) as e:
+            # The file path is not a valid image file path
+            return False
+
     @staticmethod
     def simplePrompt(inputIndicator="", validator=None, default="", accept_default=False, completer=None, promptSession=None, style=None, is_password=False, bottom_toolbar=None):
         this_key_bindings = KeyBindings()
-        @this_key_bindings.add("c-q")
+        @this_key_bindings.add(*config.hotkey_exit)
         def _(event):
             buffer = event.app.current_buffer
             buffer.text = config.exit_entry
             buffer.validate_and_handle()
+        @this_key_bindings.add(*config.hotkey_cancel)
+        def _(event):
+            buffer = event.app.current_buffer
+            buffer.reset()
+        @this_key_bindings.add(*config.hotkey_insert_newline)
+        def _(event):
+            buffer = event.app.current_buffer
+            buffer.newline()
+        @this_key_bindings.add(*config.hotkey_toggle_word_wrap)
+        def _(_):
+            config.wrapWords = not config.wrapWords
+            config.saveConfig()
+            run_in_terminal(lambda: config.print3(f"Word Wrap: '{'enabled' if config.wrapWords else 'disabled'}'!"))
+        @this_key_bindings.add(*config.hotkey_toggle_response_audio)
+        def _(_):
+            if config.tts:
+                config.ttsOutput = not config.ttsOutput
+                config.saveConfig()
+                run_in_terminal(lambda: config.print3(f"Response Audio: '{'enabled' if config.ttsOutput else 'disabled'}'!"))
+        @this_key_bindings.add(*config.hotkey_voice_entry)
+        def _(event):
+            # reference: https://github.com/Uberi/speech_recognition/blob/master/examples/microphone_recognition.py
+            def voiceTyping():
+                r = sr.Recognizer()
+                with sr.Microphone() as source:
+                    if config.voiceTypingNotification:
+                        TTSUtil.playAudioFilePygame(os.path.join(packageFolder, "audio", "notification1_mild.mp3"))
+                    #run_in_terminal(lambda: config.print2("Listensing to your voice ..."))
+                    if config.voiceTypingAdjustAmbientNoise:
+                        r.adjust_for_ambient_noise(source)
+                    audio = r.listen(source)
+                if config.voiceTypingNotification:
+                    TTSUtil.playAudioFilePygame(os.path.join(packageFolder, "audio", "notification2_mild.mp3"))
+                #run_in_terminal(lambda: config.print2("Processing to your voice ..."))
+                if config.voiceTypingPlatform == "google":
+                    # recognize speech using Google Speech Recognition
+                    try:
+                        # check google.recognize_legacy in SpeechRecognition package
+                        # check availabl languages at: https://cloud.google.com/speech-to-text/docs/speech-to-text-supported-languages
+                        # config.voiceTypingLanguage should be code list in column BCP-47 at https://cloud.google.com/speech-to-text/docs/speech-to-text-supported-languages
+                        return r.recognize_google(audio, language=config.voiceTypingLanguage)
+                    except sr.UnknownValueError:
+                        #return "[Speech unrecognized!]"
+                        return ""
+                    except sr.RequestError as e:
+                        return "[Error: {0}]".format(e)
+                elif config.voiceTypingPlatform == "googlecloud" and os.environ["GOOGLE_APPLICATION_CREDENTIALS"] and "Speech-to-Text" in config.enabledGoogleAPIs:
+                    # recognize speech using Google Cloud Speech
+                    try:
+                        # check availabl languages at: https://cloud.google.com/speech-to-text/docs/speech-to-text-supported-languages
+                        # config.voiceTypingLanguage should be code list in column BCP-47 at https://cloud.google.com/speech-to-text/docs/speech-to-text-supported-languages
+                        return r.recognize_google_cloud(audio, language=config.voiceTypingLanguage, credentials_json=config.google_cloud_credentials)
+                    except sr.UnknownValueError:
+                        #return "[Speech unrecognized!]"
+                        return ""
+                    except sr.RequestError as e:
+                        return "[Error: {0}]".format(e)
+                elif config.voiceTypingPlatform == "whisper":
+                    # recognize speech using whisper
+                    try:
+                        # check availabl languages at: https://github.com/openai/whisper/blob/main/whisper/tokenizer.py
+                        # config.voiceTypingLanguage should be uncapitalized full language name like "english" or "chinese"
+                        return r.recognize_whisper(audio, model=config.voiceTypingWhisperEnglishModel if config.voiceTypingLanguage == "english" else "large", language=config.voiceTypingLanguage)
+                    except sr.UnknownValueError:
+                        return ""
+                    except sr.RequestError as e:
+                        return "[Error]"
+
+            if config.pyaudioInstalled:
+                buffer = event.app.current_buffer
+                buffer.text = f"{buffer.text}{' ' if buffer.text else ''}{voiceTyping()}"
+                if config.voiceTypingAutoComplete:
+                    buffer.validate_and_handle()
+                else:
+                    buffer.cursor_position = buffer.cursor_position + buffer.document.get_end_of_line_position()
+            else:
+                run_in_terminal(lambda: config.print2("Install PyAudio first to enable voice entry!"))
+
         if hasattr(config, "currentMessages"):
-            @this_key_bindings.add("c-g")
+            from letmedoit.utils.prompt_shared_key_bindings import prompt_shared_key_bindings
+            from letmedoit.utils.prompt_multiline_shared_key_bindings import prompt_multiline_shared_key_bindings
+            @this_key_bindings.add(*config.hotkey_launch_pager_view)
             def _(_):
                 config.launchPager()
             # additional key binding
@@ -161,7 +273,7 @@ class HealthCheck:
                 conditional_prompt_multiline_shared_key_bindings,
             ])
         else:
-            @this_key_bindings.add("c-n")
+            @this_key_bindings.add(*config.hotkey_new)
             def _(event):
                 buffer = event.app.current_buffer
                 config.defaultEntry = buffer.text
@@ -179,7 +291,7 @@ class HealthCheck:
         userInput = inputPrompt(
             inputIndicator,
             key_bindings=this_key_bindings,
-            bottom_toolbar=bottom_toolbar if bottom_toolbar is not None else f" [ctrl+q] {config.exit_entry}",
+            bottom_toolbar=bottom_toolbar if bottom_toolbar is not None else f""" {str(config.hotkey_exit).replace("'", "")} {config.exit_entry}""",
             #enable_system_prompt=True,
             swap_light_and_dark_colors=Condition(lambda: not config.terminalResourceLinkColor.startswith("ansibright")),
             style=style,
@@ -208,7 +320,7 @@ class HealthCheck:
         return style_from_pygments_cls(get_style_by_name(theme))
 
     @staticmethod
-    def getFiles():
+    def getLocalStorage():
         apps = {
             "myhand": ("MyHand", "MyHand Bot"),
             "letmedoit": ("LetMeDoIt", "LetMeDoIt AI"),
@@ -265,7 +377,7 @@ class HealthCheck:
         from chromadb.utils import embedding_functions
         embeddingModel = embeddingModel if embeddingModel is not None else config.embeddingModel
         if embeddingModel in ("text-embedding-3-large", "text-embedding-3-small", "text-embedding-ada-002"):
-            return embedding_functions.OpenAIEmbeddingFunction(api_key=config.openaiApiKey, model_name="text-embedding-ada-002")
+            return embedding_functions.OpenAIEmbeddingFunction(api_key=config.openaiApiKey, model_name=embeddingModel)
         return embedding_functions.SentenceTransformerEmbeddingFunction(model_name=embeddingModel) # support custom Sentence Transformer Embedding models by modifying config.embeddingModel
 
     @staticmethod
@@ -294,23 +406,6 @@ class HealthCheck:
             oai_config_list.append({"model": model, "api_key": config.openaiApiKey})
         os.environ["OAI_CONFIG_LIST"] = json.dumps(oai_config_list)
 
-    @staticmethod
-    def saveConfig():
-        #print(configFile)
-        with open(configFile, "w", encoding="utf-8") as fileObj:
-            #print(dir(config))
-            for name in dir(config):
-                excludeConfigList = [
-                    "isTermux",
-                ]
-                if not name.startswith("__") and not name in excludeConfigList:
-                    try:
-                        value = eval(f"config.{name}")
-                        if not callable(value) and not str(value).startswith("<"):
-                            fileObj.write("{0} = {1}\n".format(name, pprint.pformat(value)))
-                    except:
-                        pass
-
     # The following method was modified from source:
     # https://github.com/openai/openai-cookbook/blob/main/examples/How_to_count_tokens_with_tiktoken.ipynb
     @staticmethod
@@ -326,6 +421,8 @@ class HealthCheck:
             encoding = tiktoken.get_encoding("cl100k_base")
         if model in {
                 "gpt-3.5-turbo",
+                "gpt-3.5-turbo-0125",
+                "gpt-3.5-turbo-1106",
                 "gpt-3.5-turbo-0613",
                 "gpt-3.5-turbo-16k",
                 "gpt-3.5-turbo-16k-0613",
