@@ -123,6 +123,13 @@ def add_calendar_event(function_args):
     return "Done!"
 
 functionSignature = {
+    "intent": [
+        "arrange activities",
+        "access to internet real-time information",
+    ],
+    "examples": [
+        "Add a calender event",
+    ],
     "name": "add_calendar_event",
     "description": "add calendar event",
     "parameters": {
@@ -162,4 +169,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="add_calendar_event", signature=functionSignature, method=add_calendar_event)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=add_calendar_event)
\ No newline at end of file
