@@ -18,6 +18,12 @@ def analyze_images_with_gemini(function_args):
     return "[INVALID]" if not answer else ""
 
 functionSignature = {
+    "intent": [
+        "analyze files",
+    ],
+    "examples": [
+        "analyze image with Gemini",
+    ],
     "name": "analyze_images_with_gemini",
     "description": "Use Gemini Pro Vision to describe or analyze images",
     "parameters": {
@@ -36,5 +42,5 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="analyze_images_with_gemini", signature=functionSignature, method=analyze_images_with_gemini)
+config.addFunctionCall(signature=functionSignature, method=analyze_images_with_gemini)
 config.inputSuggestions.append("Ask Gemini Pro Vision to describe this image in detail: ")
\ No newline at end of file
