@@ -18,6 +18,12 @@ def ask_chatgpt(function_args):
     return ""
 
 functionSignature = {
+    "intent": [
+        "ask a chatbot",
+    ],
+    "examples": [
+        "Ask ChatGPT about",
+    ],
     "name": "ask_chatgpt",
     "description": "Ask ChatGPT to chat or provide information",
     "parameters": {
@@ -32,5 +38,5 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="ask_chatgpt", signature=functionSignature, method=ask_chatgpt)
+config.addFunctionCall(signature=functionSignature, method=ask_chatgpt)
 config.inputSuggestions.append("Ask ChatGPT: ")
\ No newline at end of file
