@@ -17,6 +17,12 @@ def ask_codey(function_args):
     return ""
 
 functionSignature = {
+    "intent": [
+        "ask a chatbot",
+    ],
+    "examples": [
+        "Ask Codey about",
+    ],
     "name": "ask_codey",
     "description": "Ask Codey for information about coding",
     "parameters": {
@@ -31,5 +37,5 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="ask_codey", signature=functionSignature, method=ask_codey)
+config.addFunctionCall(signature=functionSignature, method=ask_codey)
 config.inputSuggestions.append("Ask Codey: ")
\ No newline at end of file
