@@ -17,6 +17,12 @@ def ask_gemini_pro(function_args):
     return ""
 
 functionSignature = {
+    "intent": [
+        "ask a chatbot",
+    ],
+    "examples": [
+        "Ask Gemini about",
+    ],
     "name": "ask_gemini_pro",
     "description": "Ask Gemini Pro to chat or provide information",
     "parameters": {
@@ -31,5 +37,5 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="ask_gemini_pro", signature=functionSignature, method=ask_gemini_pro)
+config.addFunctionCall(signature=functionSignature, method=ask_gemini_pro)
 config.inputSuggestions.append("Ask Gemini Pro: ")
\ No newline at end of file
