@@ -17,6 +17,12 @@ def ask_palm2(function_args):
     return ""
 
 functionSignature = {
+    "intent": [
+        "ask a chatbot",
+    ],
+    "examples": [
+        "Ask PaLM about",
+    ],
     "name": "ask_palm2",
     "description": "Ask PaLM 2 to chat or provide information",
     "parameters": {
@@ -31,5 +37,5 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="ask_palm2", signature=functionSignature, method=ask_palm2)
+config.addFunctionCall(signature=functionSignature, method=ask_palm2)
 config.inputSuggestions.append("Ask PaLM 2: ")
\ No newline at end of file
