@@ -40,6 +40,12 @@ def heal_python(function_args):
         return traceback.format_exc()
 
 functionSignature = {
+    "intent": [
+        "generate code",
+    ],
+    "examples": [
+        "Fix python code",
+    ],
     "name": "heal_python",
     "description": "Fix python code if both original code and traceback error are provided",
     "parameters": {
@@ -69,4 +75,4 @@ persistentConfigs = (
 )
 config.setConfig(persistentConfigs)
 
-config.addFunctionCall(name="heal_python", signature=functionSignature, method=heal_python)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=heal_python)
\ No newline at end of file
