@@ -25,6 +25,14 @@ def build_agents(function_args):
     return ""
 
 functionSignature = {
+    "intent": [
+        "ask an auto assistant",
+        "create content",
+    ],
+    "examples": [
+        "Ask autobuilder to",
+        "Create a team of agents / assistants to",
+    ],
     "name": "build_agents",
     "description": "build a group of AI assistants or agents to execute a complicated task that other functions cannot resolve",
     "parameters": {
@@ -43,4 +51,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="build_agents", signature=functionSignature, method=build_agents)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=build_agents)
\ No newline at end of file
