@@ -24,6 +24,12 @@ def create_map(function_args):
     return ""
 
 functionSignature = {
+    "intent": [
+        "create content",
+    ],
+    "examples": [
+        "Create a map",
+    ],
     "name": "create_map",
     "description": f'''Create maps''',
     "parameters": {
@@ -38,4 +44,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="create_map", signature=functionSignature, method=create_map)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=create_map)
\ No newline at end of file
