@@ -24,6 +24,12 @@ def create_qrcode(function_args):
     return ""
 
 functionSignature = {
+    "intent": [
+        "create content",
+    ],
+    "examples": [
+        "Create a QR code",
+    ],
     "name": "create_qrcode",
     "description": f'''Create QR code''',
     "parameters": {
@@ -38,4 +44,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="create_qrcode", signature=functionSignature, method=create_qrcode)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=create_qrcode)
\ No newline at end of file
