@@ -15,6 +15,13 @@ def datetimes(function_args):
     return information
 
 functionSignature = {
+    "intent": [
+        "access to internet real-time information",
+    ],
+    "examples": [
+        "What is the current time in Hong Kong?",
+        "Tell me the date of the upcoming Friday?",
+    ],
     "name": "datetimes",
     "description": f'''Get information about dates and times''',
     "parameters": {
@@ -29,4 +36,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="datetimes", signature=functionSignature, method=datetimes)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=datetimes)
\ No newline at end of file
