@@ -37,6 +37,12 @@ def edit_text(function_args):
         return "Finished! Text editor closed!"
 
 functionSignature = {
+    "intent": [
+        "change files",
+    ],
+    "examples": [
+        "Edit test.txt",
+    ],
     "name": "edit_text",
     "description": f'''Edit text files with extensions: '*.{"', '*.".join(config.textFileExtensions)}'.''',
     "parameters": {
@@ -51,4 +57,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="edit_text", signature=functionSignature, method=edit_text)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=edit_text)
\ No newline at end of file
