@@ -20,6 +20,12 @@ def install_package(function_args):
 
 # Function Signature
 functionSignature = {
+    "intent": [
+        "installation",
+    ],
+    "examples": [
+        "Install package",
+    ],
     "name": "install_package",
     "description": f'''Install python package''',
     "parameters": {
@@ -35,4 +41,4 @@ functionSignature = {
 }
 
 # Integrate the signature and method into LetMeDoIt AI
-config.addFunctionCall(name="install_package", signature=functionSignature, method=install_package)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=install_package)
\ No newline at end of file
