@@ -32,6 +32,12 @@ def integrate_google_searches(function_args):
     return json.dumps(info)
 
 functionSignature = {
+    "intent": [
+        "access to internet real-time information",
+    ],
+    "examples": [
+        "Search internet",
+    ],
     "name": "integrate_google_searches",
     "description": "Search internet for keywords when ChatGPT lacks information or when user ask about latest updates",
     "parameters": {
@@ -46,4 +52,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="integrate_google_searches", signature=functionSignature, method=integrate_google_searches)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=integrate_google_searches)
\ No newline at end of file
