@@ -17,6 +17,12 @@ def manipulate_files(function_args):
     return SharedUtil.showAndExecutePythonCode(code)
 
 functionSignature = {
+    "intent": [
+        "change files",
+    ],
+    "examples": [
+        "Edit test.txt",
+    ],
     "name": "manipulate_files",
     "description": f'''Manipulate files, such as opening, launching, navigation, renaming, editing, removal, conversion, etc.''',
     "parameters": {
@@ -31,4 +37,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="manipulate_files", signature=functionSignature, method=manipulate_files)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=manipulate_files)
\ No newline at end of file
