@@ -18,6 +18,14 @@ def open_browser(function_args):
 
 # Function Signature
 functionSignature = {
+    "intent": [
+        "access to internet real-time information",
+    ],
+    "examples": [
+        "Open web browser",
+        "Open website",
+        "Open https://letmedoit.ai",
+    ],
     "name": "open_browser",
     "description": f'''Open an url with default web browser''',
     "parameters": {
@@ -33,5 +41,5 @@ functionSignature = {
 }
 
 # Integrate the signature and method into LetMeDoIt AI
-config.addFunctionCall(name="open_browser", signature=functionSignature, method=open_browser)
+config.addFunctionCall(signature=functionSignature, method=open_browser)
 config.inputSuggestions.append("Open url: ")
\ No newline at end of file
