@@ -26,6 +26,12 @@ def pronunce_words(function_args):
     return "Finished! Speech engine closed!"
 
 functionSignature = {
+    "intent": [
+        "interact with user",
+    ],
+    "examples": [
+        "Pronunce",
+    ],
     "name": "pronunce_words",
     "description": "pronounce words or sentences",
     "parameters": {
@@ -45,5 +51,5 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="pronunce_words", signature=functionSignature, method=pronunce_words)
+config.addFunctionCall(signature=functionSignature, method=pronunce_words)
 config.inputSuggestions.append("pronunce ")
