@@ -21,6 +21,12 @@ def search_finance(function_args):
     return ""
 
 functionSignature = {
+    "intent": [
+        "access to internet real-time information",
+    ],
+    "examples": [
+        "Check stock price",
+    ],
     "name": "search_finance",
     "description": f'''Search or analyze financial data. Use this function ONLY WHEN package yfinance is useful to resolve my request''',
     "parameters": {
@@ -35,4 +41,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="search_finance", signature=functionSignature, method=search_finance)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=search_finance)
\ No newline at end of file
