@@ -93,6 +93,13 @@ def send_email(function_args):
     return "Done!"
 
 functionSignature = {
+    "intent": [
+        "arrange activities",
+        "access to internet real-time information",
+    ],
+    "examples": [
+        "Send email",
+    ],
     "name": "send_email",
     "description": "send email",
     "parameters": {
@@ -120,4 +127,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="send_email", signature=functionSignature, method=send_email)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=send_email)
\ No newline at end of file
