@@ -20,6 +20,13 @@ def send_whatsapp(function_args):
     return "Done!"
 
 functionSignature = {
+    "intent": [
+        "arrange activities",
+        "access to internet real-time information",
+    ],
+    "examples": [
+        "Send WhatsApp",
+    ],
     "name": "send_whatsapp",
     "description": f'''Send WhatsApp messages''',
     "parameters": {
@@ -38,4 +45,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="send_whatsapp", signature=functionSignature, method=send_whatsapp)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=send_whatsapp)
\ No newline at end of file
