@@ -19,4 +19,4 @@ def convertToTraditionalChinese(text):
     else:
         return text
 
-config.chatGPTTransformers.append(convertToTraditionalChinese)
\ No newline at end of file
+config.outputTransformers.append(convertToTraditionalChinese)
\ No newline at end of file
