@@ -20,6 +20,12 @@ def solve_math(function_args):
     return ""
 
 functionSignature = {
+    "intent": [
+        "resolve a math question",
+    ],
+    "examples": [
+        "Calculate",
+    ],
     "name": "solve_math",
     "description": "solve math problems",
     "parameters": {
@@ -34,4 +40,4 @@ functionSignature = {
     },
 }
 
-config.addFunctionCall(name="solve_math", signature=functionSignature, method=solve_math)
\ No newline at end of file
+config.addFunctionCall(signature=functionSignature, method=solve_math)
\ No newline at end of file
